Package: HTSFilter
Type: Package
Title: Filter replicated high-throughput transcriptome sequencing data
Version: 1.52.0
Date: 2020-12-04
Authors@R: c(
  person(given="Andrea", 
         family="Rau", 
         email="andrea.rau@inrae.fr", 
         role=c("cre", "aut"), 
         comment = c(ORCID = "0000-0001-6469-488X")),
  person(given="Melina",
         family="Gallopin",
         role="ctb"),
  person(given="Gilles",
         family="Celeux",
         role="ctb"),
  person(given="Florence",
         family="Jaffrézic",
         role="ctb"))
Depends: R (>= 4.0.0)
Imports: edgeR, DESeq2, BiocParallel, Biobase, utils, stats, grDevices,
        graphics, methods
Suggests: EDASeq, testthat, knitr, rmarkdown, BiocStyle
Description: This package implements a filtering procedure for
    replicated transcriptome sequencing data based on a global
    Jaccard similarity index in order to identify genes with low,
    constant levels of expression across one or more experimental
    conditions.
License: Artistic-2.0
LazyLoad: yes
biocViews: Sequencing, RNASeq, Preprocessing, DifferentialExpression,
        GeneExpression, Normalization, ImmunoOncology
RoxygenNote: 7.1.1
VignetteBuilder: knitr
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/HTSFilter
git_branch: RELEASE_3_23
git_last_commit: 40d8543
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:39:11 UTC; biocbuild
Author: Andrea Rau [cre, aut] (ORCID: <https://orcid.org/0000-0001-6469-488X>),
  Melina Gallopin [ctb],
  Gilles Celeux [ctb],
  Florence Jaffrézic [ctb]
Maintainer: Andrea Rau <andrea.rau@inrae.fr>
